package xyz.thastertyn.Scrape;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.regex.Pattern;

import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import xyz.thastertyn.Tuples.Pair;
import xyz.thastertyn.Types.Choice;
import xyz.thastertyn.Types.Option;
import xyz.thastertyn.Types.Options;
import xyz.thastertyn.Types.Predmet;
import xyz.thastertyn.Types.Znamka;

public class Znamky {

	private boolean wasDownloaded;

	// schoolYear, schoolYearId
	private ArrayList<Predmet> predmety = new ArrayList<>();

	// int znaci id roku, boolean jestli je jen prvni nebo i druhe pololeti
	private Options schoolYearOptions = new Options("Skolni R.");
	private Options schoolHalfYearOptions = new Options("Pololeti");

	public void downloadZnamky() throws UnknownHostException, IOException
	{
		download("https://www.spsejecna.cz/score/student");
	}
	
	public void downloadZnamky(Choice choice) throws UnknownHostException, IOException
	{
		download(String.format(
			"https://www.spsejecna.cz/score/student?schoolYearId=%s&schoolYearHalfId=%s", 
			choice.getChoices().get(0), 
			choice.getChoices().get(1)));
	}

	private void download(String url) throws UnknownHostException, IOException
	{
		//String url = 
		Document znamkyDokumentHTML = Downloader.download(url).get();

		// Predmety ulozene v <tr>
		Elements[] radkyPredmetuHTML = znamkyDokumentHTML
			.select("table.score")
			.select("tr")
			.stream()
			.map(Element::children)
			.toArray(Elements[]::new);

		int subjectIndex = 0;

		for(int i = 1; i < radkyPredmetuHTML.length; i++)
		{
			String plnyNazevPredmetu = radkyPredmetuHTML[i].get(0).text();

			String jmenoPredmetu = Pattern
				.compile("\\((.*?)\\)")
				.matcher(plnyNazevPredmetu)
				.results()
				.findFirst()
				.map(m -> m.group(1))
				.orElse(plnyNazevPredmetu);

			predmety.add(new Predmet(jmenoPredmetu));

			for(Element znamkaElement : radkyPredmetuHTML[i].get(1).select("a.score"))
			{
				int znamka;
				boolean jeMala = false;

				String textZnamky = znamkaElement.select("span.value").text();

				znamka = textZnamky.matches("\\d") ?
					Integer.parseInt(textZnamky) :
					-1; // Nejspis se jedna o N (Nehodnocen)

				// Mala znamka se bude pocitat jako polovicni vaha
				jeMala = znamkaElement.hasClass("scoreSmall");

				predmety.get(subjectIndex).addZnamka(new Znamka(znamka, jeMala, textZnamky));
			}

			subjectIndex++;
		}

		Element optionsPanel = znamkyDokumentHTML.selectFirst("form.listConfigure");
		
		Elements skolniRoky = optionsPanel.select("select[id=schoolYearId]").select("option");
		Elements pololeti = optionsPanel.select("select[id=schoolYearHalfId]").select("option");

		for(Element e : skolniRoky)
		{
			schoolYearOptions.addOption(new Option(e.text(), e.attr("value")));
		}

		for(Element e : pololeti)
		{
			schoolHalfYearOptions.addOption(new Option(e.text(), e.attr("value")));
		}

		wasDownloaded = true;
	}
	
	public ArrayList<Predmet> getPredmety()
	{
		return predmety;
	}

	public Pair<Options, Options> getOptions()
	{
		return new Pair<Options, Options>(schoolYearOptions, schoolHalfYearOptions);
	}

	@Override
	public String toString()
	{
		return (wasDownloaded) ?
			"All up and ready for use" :
			"Nothing downloaded yet";
	}
}