package xyz.thastertyn.UserInterface.Content;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import com.googlecode.lanterna.TextColor;
import com.googlecode.lanterna.TextColor.ANSI;
import com.googlecode.lanterna.graphics.SimpleTheme;
import com.googlecode.lanterna.gui2.Direction;
import com.googlecode.lanterna.gui2.GridLayout;
import com.googlecode.lanterna.gui2.Label;
import com.googlecode.lanterna.gui2.LayoutData;
import com.googlecode.lanterna.gui2.LinearLayout;
import com.googlecode.lanterna.gui2.Panel;
import com.googlecode.lanterna.gui2.WindowBasedTextGUI;

import xyz.thastertyn.Types.Choice;
import xyz.thastertyn.Types.Predmet;
import xyz.thastertyn.Types.Znamka;
import xyz.thastertyn.UserInterface.UpdateListener;
import xyz.thastertyn.UserInterface.Dialogs.OptionsDialog;

public class Znamky extends JecnaContent {

	private final TextColor.RGB VYBORNY = new TextColor.RGB(85,212,0);
	private final TextColor.RGB CHVALITEBNY = new TextColor.RGB(196,224,80);
	private final TextColor.RGB DOBRY = new TextColor.RGB(255,213,42);
	private final TextColor.RGB DOSTATECNY = new TextColor.RGB(255,102,0);
	private final TextColor.RGB NEDOSTATECNY = new TextColor.RGB(255,48,48);
	private final TextColor.RGB NEHODNOCEN = new TextColor.RGB(0,0,0);

	private final LayoutData ALIGN_LEFT = GridLayout.createLayoutData(
		GridLayout.Alignment.BEGINNING,
		GridLayout.Alignment.BEGINNING,
		false,
		false);

	private final LayoutData ALIGN_RIGHT = GridLayout.createLayoutData(
		GridLayout.Alignment.END,
		GridLayout.Alignment.BEGINNING,
		true,
		false);
	
	private Panel mainPanel;
	private Label borderLabel = new Label("Znamky");
	private xyz.thastertyn.Scrape.Znamky znamky;

	private UpdateListener listener;
	
	private boolean hasStarted = false;

	@Override
	public void downloadDefault() throws IOException
	{
		download(null);
	}

	@Override
	protected void download(Choice choice) throws IOException
	{
		znamky = new xyz.thastertyn.Scrape.Znamky();
		if(choice != null)
		{
			znamky.downloadZnamky(choice);
		}else{
			znamky.downloadZnamky();
		}
		
		setGUI();
		listener.updatePanel();

		hasStarted = true;
	}

	@Override
	protected void setGUI()
	{
		mainPanel = new Panel()
		.setLayoutManager(new GridLayout(3));
	
		ArrayList<Predmet> subjects = znamky.getPredmety();
			
		HashMap<Integer, SimpleTheme> colors = new HashMap<>();
		colors.put(1, new SimpleTheme(ANSI.BLACK, VYBORNY));
		colors.put(2, new SimpleTheme(ANSI.BLACK, CHVALITEBNY));
		colors.put(3, new SimpleTheme(ANSI.BLACK, DOBRY));
		colors.put(4, new SimpleTheme(ANSI.BLACK, DOSTATECNY));
		colors.put(5, new SimpleTheme(ANSI.BLACK, NEDOSTATECNY));
		colors.put(-1, new SimpleTheme(ANSI.WHITE, NEHODNOCEN));

		// Sloupec pro jmena predmetu
		Panel jmemaPredmetu = new Panel()
		.setLayoutManager(new LinearLayout(Direction.VERTICAL))
		.setLayoutData(ALIGN_LEFT)
		.addTo(mainPanel);
	
		// Sloupec pro znamky z predmetu
		Panel znamky = new Panel()
		.setLayoutManager(new LinearLayout(Direction.VERTICAL))
		.setLayoutData(ALIGN_LEFT)
		.addTo(mainPanel);

		// Sloupec pro vyslednou znamku
		Panel vysledneZnamky = new Panel()
		.setLayoutManager(new LinearLayout(Direction.VERTICAL))
		.setLayoutData(ALIGN_RIGHT)
		.addTo(mainPanel);

		for(Predmet predmet : subjects)
		{
			Panel jednotliveZnamky = new Panel()
			.setLayoutManager(new LinearLayout(Direction.HORIZONTAL))
			.addTo(znamky);

			if(predmet.getZnamky().isEmpty())
			{
				jednotliveZnamky.addComponent(new Label(""));
			}

			for(Znamka znamka : predmet.getZnamky())
			{
				Label znamkaLabel = new Label(znamka.getText());
				znamkaLabel.setTheme(colors.get(znamka.getZnamka()));
				jednotliveZnamky.addComponent(znamkaLabel);
			}
			
			double prumer = predmet.getVyslednaZnamka();

			Label vysl = new Label(String.format("%.2f", prumer));
			
			vysl.setTheme(colors.get((int) Math.round(prumer)));
			
			jmemaPredmetu.addComponent(new Label(predmet.getJmenoPredmetu()));
			vysledneZnamky.addComponent(vysl);
		}
		mainPanel.invalidate();
	}


	@Override
	public Panel getPanel()
	{
		return mainPanel;
	}

	@Override
	public boolean hasStarted()
	{
		return hasStarted;
	}

	@Override
	public Label getLabel()
	{
		return borderLabel;
	}

	@Override
	public void showOptions(WindowBasedTextGUI textGUI) throws IOException {
		OptionsDialog d = new OptionsDialog(znamky.getOptions().getValue0(), znamky.getOptions().getValue1());

		Choice c = d.showDialog(textGUI);

		if(c != null)
		{
			download(c);
		}
	}

	@Override
	public void setUpdateListener(UpdateListener updateListener) {
		this.listener = updateListener;
	}
}